﻿using System;
using System.Net;
using System.Net.Sockets;

namespace EonaCat.Sockets
{
    public delegate void SendBytesTo(byte[] data, EndPoint point);

    public class Client : IDisposable
    {
        /// <summary>
        /// Unique client id.
        /// </summary>
        public string Guid { get; set; }

        /// <summary>
        /// Client socket.
        /// </summary>
        public Socket Socket { get; set; }

        /// <summary>
        /// Receive buffer.
        /// </summary>
        public byte[] Buffer { get; set; }

        /// <summary>
        /// Buffer size
        /// </summary>
        public static int BufferSize { get; set; } = 8 * 1024; // 8KB

        /// <summary>
        /// Socket connection state.
        /// </summary>
        public bool IsConnected => !((Socket.Poll(1000, SelectMode.SelectRead) && (Socket.Available == 0)) || !Socket.Connected);
        /// <summary>
        /// Client ip.
        /// </summary>
        public IPAddress RemoteIP => (Socket.RemoteEndPoint as IPEndPoint)?.Address;
        /// <summary>
        /// Client port.
        /// </summary>
        public int RemotePort => (Socket.RemoteEndPoint as IPEndPoint).Port;

        public event SendBytesTo SendData;

        public Client()
        {
            Buffer = new byte[BufferSize];
        }

        public void Dispose()
        {
            Socket.Close();
        }

        /// <summary>
        /// Send data to the client.
        /// </summary>
        /// <param name="data">Data for sending</param>
        public void Send(byte[] data)
        {
            switch (Socket.SocketType)
            {
                case SocketType.Stream:
                    Socket.BeginSend(data, 0, data.Length, 0, EndSend, null);
                    break;
                case SocketType.Dgram when SendData != null:
                    SendData?.Invoke(data, Socket.RemoteEndPoint);
                    break;
                case SocketType.Dgram:
                    Socket.BeginSendTo(data, 0, data.Length, 0, Socket.RemoteEndPoint, EndSendTo, null);
                    break;
            }
        }

        private void EndSend(IAsyncResult ar)
        {
            Socket.EndSend(ar);
        }

        private void EndSendTo(IAsyncResult ar)
        {
            Socket.EndSendTo(ar);
        }
    }
}